﻿/* 线程类 继承 QThread
 * run() 不同 task操作
 */
#include "filethread.h"
const qint64 fileblocksize = 1024*1024*1;
/*
 * 分块下载：本地创建temp文件夹，每个下载任务会在temp中创建一个文件夹A(文件夹名称：MD5加密的文件uuid)，
 * 在文件夹A下创建每块分段文件（文件名称：temp+分块序号），所有分块文件下载完成后进行合并，合并完成后立即删除缓存
 * 文件夹A，下载失败也要删除缓存文件（有可能是分块文件缺失导致，删除之后可以重新下载）
 *
*/
const QString downloadTempFileName = NETDISK_DOWNLOADTEMPFILE_NAME;

filethread::filethread(): m_bquit(false),m_ndownloadblock(0)
{
    disktool = netdisksession::getInstance();
    userDBTool = UserDbSession::getInstance();
    m_downloadBlockNumber = 0;
    m_isFinish = 0;
}

filethread::~filethread()
{

}

void filethread::addTask(const loadfiletask *task)
{
    m_strfilename = task->filename;
    m_fileType = task->fileType;
    m_fileToUuid = task->toUuid;
    m_strfileuuid = task->fileuuid;
    m_strfilelocalpath = task->filelocalpath;
    m_nType = task->loadtype;
    m_nfilesize = task->filesize;
    m_uploadedSize = task->loadedsize;
    m_filelocaldirpath = task->filelocaldirpath;

    m_uploadFileInfo.filename = m_strfilename;
    m_uploadFileInfo.fileType = m_fileType;
    m_uploadFileInfo.filesize = QString("%1").arg(m_nfilesize);
    m_uploadFileInfo.toFileUuid = m_fileToUuid;

    QString fileImageName = disktool->getFileTypeImage(m_fileType);
    m_uploadFileInfo.showSize = disktool->getShowSizeWithFileSize(m_uploadFileInfo.filesize);
    //m_uploadFileInfo.radAbsPath = fileinfor.value("radAbsPath").toString();//上传完成后未返回
    //m_uploadFileInfo.radSpecialType = fileinfor.value("radSpecialType").toString().toInt();//上传完成后未返回
    m_uploadFileInfo.isdirectory =0;
    m_uploadFileInfo.filename=m_strfilename+m_fileType;
    m_uploadFileInfo.name=m_strfilename;
    //m_uploadFileInfo.fileMd5=fileinfor.value("md5").toString();
    m_uploadFileInfo.fileMd5 = m_strfilemd5;
    m_uploadFileInfo.toFileUuid = m_fileToUuid;
    m_uploadFileInfo.filetypeimage =":/image/iconmin/"+fileImageName;
    m_uploadFileInfo.filetypeMaximage =":/image/iconmax/"+fileImageName;
    m_uploadFileInfo.isRecycleFile = 0;

    if(m_uploadedSize > 0) {
        if(m_uploadedSize >= fileblocksize) {
            m_uploadedBlock = m_uploadedSize/fileblocksize;
        }
    } else {
        m_uploadedSize = 0;
        m_uploadedBlock = 0;
    }
    if(m_nfilesize <= 0) {
        m_nfilesize = 0;
    }
    if(fileblocksize > 0) {
        m_ndownloadblock = task->filesize / fileblocksize;
    } else {
        m_ndownloadblock = 0;
    }
    if(m_ndownloadblock <0) {
        m_ndownloadblock = 0;
    }
}

void filethread::run()
{
    if(m_nType == 1) {
        uploadfile();
    } else if(m_nType == 2){
        downloadfile();
    }
    m_bquit = true;
}

void filethread::getCurrentTime()
{
    QDateTime curDateTime=QDateTime::currentDateTime();
    QString time000 = curDateTime.toString("yyyy-MM-dd hh:mm:ss");
    m_uploadFileInfo.filedatetime = time000;
}

void filethread::uploadfile()
{
    //上传校验文件 文件名 MD5
    int nreturncode = 0;
    if(!m_bquit)
        nreturncode  = beforecheckfileinfo();
    switch (nreturncode) {
    case 1: {
        //上传文件分片
        uploadblock();
        //完成校验
        if(!m_bquit)
            checksuccessuploadfile();
        break;
    } case 6: {
        m_uploadedSize = 0;
        m_isFinish = 1;
        //秒传成功
        emit updatepoolloadfileprocess(1, m_strfileuuid, m_nfilesize,m_nfilesize, 1,2,m_uploadFileInfo);
        break;
    }  case 2: {
        m_uploadedSize = 0;
        m_isFinish = 1;
        //容量不足
        emit updatepoolloadfilestatus(1, m_strfileuuid, 3);
        break;
    }  case 4: {
        m_isFinish = 1;
        //文件已删除
        m_bquit = true;
        emit updatepoolloadfilestatus(1, m_strfileuuid, 4);
        break;
    } default : {
        m_uploadedSize = 0;
        m_isFinish = 1;
        emit updatepoolloadfilestatus(1, m_strfileuuid, 2);
        break;

    }
    }
}

int filethread::beforecheckfileinfo()
{
    QVariant cook;
    cook.setValue(disktool->getcookies());

    QFileInfo fileinfor(m_strfilelocalpath);
    if(!fileinfor.exists()) {
        return 4;
    }

    m_nfilesize = fileinfor.size();

    QString strfilemd5;
    getfilemd5(m_strfilelocalpath, strfilemd5);
    m_strfilemd5 = strfilemd5;

    m_uploadFileInfo.fileMd5 = strfilemd5;
    m_uploadFileInfo.filesize = QString("%1").arg(m_nfilesize);
    m_uploadFileInfo.showSize = disktool->getShowSizeWithFileSize(m_uploadFileInfo.filesize);

    QString url=NetDiskNetWork_VALUE_checkBeforeUpload;
    QString string000 = QString("md5=%1&fileName=%2&fileSize=%3")
            .arg(m_strfilemd5).arg(m_strfilename)
            .arg(QString::number(m_nfilesize));
    if(!m_fileToUuid.isEmpty()) {
        string000 = QString("%1&dirUuid=%2").arg(string000).arg(m_fileToUuid);
    }
    QByteArray checkparameter = string000.toUtf8();
    //qDebug()<<"-----------校验是否秒传接口url="<<url<<"--------"<<checkparameter;

    QNetworkRequest netquest;

    QSslConfiguration conf = netquest.sslConfiguration();
    conf.setPeerVerifyMode(QSslSocket::VerifyNone);
    conf.setProtocol(QSsl::TlsV1SslV3);
    netquest.setSslConfiguration(conf);

    netquest.setUrl(QUrl(url));
    netquest.setHeader(QNetworkRequest::CookieHeader,cook);
    netquest.setRawHeader("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
    QNetworkReply *reply;
    QNetworkAccessManager *manager = disktool->getWorkManager();
    reply=manager->post(netquest,checkparameter);
    QEventLoop waitloop;
    connect(reply, &QNetworkReply::finished, &waitloop, &QEventLoop::quit);
    waitloop.exec(QEventLoop::ExcludeUserInputEvents);
    try {
        if (reply->error() == QNetworkReply::NoError) {
            QByteArray rec=reply->readAll();
            QString mkrec=QString::fromUtf8(rec);
            //qDebug()<<"-----------校验是否秒传接口="<<mkrec;
            if(rec.trimmed().isEmpty())
                return  0;
            QString strerror(rec);
            QJsonParseError jsError;
            QJsonDocument doc(QJsonDocument::fromJson(rec,&jsError));
            if (jsError.error !=QJsonParseError::NoError) {
                return 0;
            }
            bool success = doc.object().value("success").toBool();
            //{"success":false,"fieldErrors":{},"actionErrors":["容量不足"],"messages":[]}
            if(success) {
                QJsonObject dataMap = doc.object().value("data").toObject();
                QString msgString = dataMap.value("msg").toString();
                //qDebug()<<"-----------校验是否秒传接口="<<msgString;
                if (msgString=="秒传成功") {
                    QString fileUuid = dataMap.value("fileUuid").toString();
                    QString fileName = dataMap.value("fileName").toString();
                    m_uploadFileInfo.fileUuid = fileUuid;
                    m_uploadFileInfo.filename = fileName+m_fileType;
                    getCurrentTime();
                    return 6;
                }
                if (msgString=="校验成功") {
                    return 1;
                }
            } else {
                QJsonArray actionErrors = doc.object().value("actionErrors").toArray();
                if(actionErrors.count()>0) {
                    QString msgString = actionErrors.first().toString();
                    //qDebug()<<"-----------校验是否秒传接口="<<msgString;
                    m_failureMsg = msgString;
                    if(msgString.contains("容量不足")) {
                        return 2;
                    } else {
                        return 0;
                    }
                } else {
                    m_failureMsg = "校验失败";
                    return 0;
                }
            }
        }
    } catch (QString exception) {
         return 0;
    }
    return 0;
}

int filethread::checkuploadwhichblock(const QString &md5, const qint64 &block, const qint64& blocksize)
{
    QVariant cook;
    cook.setValue(disktool->getcookies());
    QByteArray param;
    QString url=NetDiskNetWork_VALUE_checkExcerpt;
    param=QString("md5=%1").arg(md5).toUtf8();
    param+=QString("&excerptNO=%1").arg(block).toUtf8();
    param+=QString("&excerptSize=%1").arg(blocksize);
    //qDebug()<<"-----------校验文件分块是否上传接口url="<<url<<"--------"<<param;

    QNetworkRequest netquest;
    QSslConfiguration conf = netquest.sslConfiguration();
    conf.setPeerVerifyMode(QSslSocket::VerifyNone);
    conf.setProtocol(QSsl::TlsV1SslV3);
    netquest.setSslConfiguration(conf);
    netquest.setUrl(QUrl(url));
    netquest.setRawHeader("Charsert","UTF-8");
    netquest.setRawHeader("Connection", "keep-alive");
    netquest.setHeader(QNetworkRequest::CookieHeader,cook);
    QEventLoop waitloop;
    QNetworkReply *reply;
    QNetworkAccessManager *manager = disktool->getWorkManager();

    reply=manager->post(netquest,param);
    connect(reply, &QNetworkReply::finished, &waitloop, &QEventLoop::quit);
    waitloop.exec(QEventLoop::ExcludeUserInputEvents);
    try {
        if (reply->error() == QNetworkReply::NoError)
        {
            QByteArray rec=reply->readAll();
            QString mkrec=QString::fromUtf8(rec);
            QJsonParseError jsError;
            QJsonDocument doc(QJsonDocument::fromJson(rec,&jsError));
            if (jsError.error !=QJsonParseError::NoError) {
                return 0;
            }
            QJsonObject dataMap = doc.object().value("data").toObject();
            QString msgString = dataMap.value("msg").toString();
            //qDebug()<<"------文件上传第"<<block<<"块分块,校验文件分块是否上传接口="<<msgString<<endl;
            if (msgString =="已上传") {
                return  1;
            }
        }
    } catch (QString exception) {
         return 0;
    }
    return 0;
}

int filethread::uploadblock()
{
    QVariant cook;
    cook.setValue(disktool->getcookies());
    QString url=NetDiskNetWork_VALUE_uploadEexcerpt;
    //qDebug()<<"-----------文件上传接口url="<<url;
    QNetworkRequest netquest;
    QSslConfiguration conf = netquest.sslConfiguration();
    conf.setPeerVerifyMode(QSslSocket::VerifyNone);
    conf.setProtocol(QSsl::TlsV1SslV3);
    netquest.setSslConfiguration(conf);
    netquest.setUrl(QUrl(url));
    netquest.setRawHeader("Charsert","UTF-8");
    netquest.setRawHeader("Connection", "keep-alive");
    netquest.setRawHeader("Accept", "application/json");
    //netquest.setRawHeader("Content-Type","multipart/form-data");
    //netquest.setRawHeader("Content-Type","application/x-www-form-urlencoded");
    netquest.setHeader(QNetworkRequest::CookieHeader,cook);
    QEventLoop waitloop;
    QNetworkReply *netreply;

    int nprocess = 0;
    QTime starttime = QTime::currentTime();
    int ntime = 0;
    int fileblocks = 1;
    if(fileblocksize > 0) {
        fileblocks = m_nfilesize/fileblocksize;
        if ((m_nfilesize%fileblocksize)>0)
            fileblocks +=1;
        if (fileblocks==0)
            fileblocks=1;
    }
    bool bloaded = false;
    qint64 loadsize = 0;
    qint64 i = 0;
    //i = m_uploadedBlock > 0 ? m_uploadedBlock:0;
    //loadsize = m_uploadedSize > 0 ? m_uploadedSize:0;
    QFile file(m_strfilelocalpath);
    file.open(QIODevice::ReadOnly);
    file.seek(loadsize);
    //qDebug()<<"------上传分块总共"<<fileblocks<<"块"<<endl;
    if(!file.exists()) {
        emit updatepoolloadfilestatus(1, m_strfileuuid, 4);
        m_bquit = true;
        return 4;
    }
    while ( !file.atEnd() && !m_bquit) {
        ++i;
        nprocess = 100 * i / fileblocks;
        qint64 chunksizetemp = fileblocksize;
        if(i == fileblocks) {
            qint64 residueSize = m_nfilesize % fileblocksize;
            if(residueSize > 0) {
                chunksizetemp = residueSize;
            } else {
                break;
            }
        }
        QByteArray fileblock=file.read(chunksizetemp);

        loadsize += chunksizetemp;
        int isUpload = checkuploadwhichblock(m_strfilemd5, i, chunksizetemp);
        //qDebug()<<"第"<<i<<"----------上传="<<isUpload<<endl;
        if(!bloaded && isUpload) {
            //此片段文件已上传，不在执行上传操作
            //m_uploadedSize = loadsize;
            //m_uploadedBlock = i;
            QTime endTime000 = QTime::currentTime();
            int nslapsed = starttime.msecsTo(endTime000);
            emit updatepoolloadfileprocess(1,m_strfileuuid, m_uploadedSize , m_nfilesize, nslapsed/1000, 0,m_uploadFileInfo);
            continue;
        }

        bloaded = true;

        QHttpMultiPart *multiPart = new QHttpMultiPart(QHttpMultiPart::FormDataType);
        QHttpPart parameters1;
        parameters1.setRawHeader("Content-Disposition",QString("form-data; name=\"%1\"").arg("excerptSize").toUtf8());
        parameters1.setBody(QString::number(chunksizetemp).toUtf8());
        multiPart->append(parameters1);
        QHttpPart parameters2;
        parameters2.setRawHeader("Content-Disposition",QString("form-data; name=\"%1\"").arg("excerptTotalNum").toUtf8());
        parameters2.setBody(QString::number(fileblocks).toUtf8());
        multiPart->append(parameters2);
        QHttpPart parameters3;
        parameters3.setRawHeader("Content-Disposition",QString("form-data; name=\"%1\"").arg("excerptNO").toUtf8());
        parameters3.setBody(QString::number(i).toUtf8());
        multiPart->append(parameters3);
        QHttpPart parameters4;
        parameters4.setRawHeader("Content-Disposition",QString("form-data; name=\"%1\"").arg("md5").toUtf8());
        parameters4.setBody(m_strfilemd5.toUtf8());
        multiPart->append(parameters4);
        QHttpPart parameters5;
        parameters5.setRawHeader("Content-Disposition",QString("form-data; name=\"%1\"; filename=\"%2\"").arg("file").arg(m_strfilename).toUtf8());
        parameters5.setBody(fileblock);
        multiPart->append(parameters5);

        netreply=disktool->getWorkManager()->post(netquest,multiPart);
        connect(netreply, &QNetworkReply::finished, &waitloop, &QEventLoop::quit);
        waitloop.exec(QEventLoop::ExcludeUserInputEvents);
        if (netreply->error()==QNetworkReply::NoError) {
            QByteArray rec=netreply->readAll();
            QString mkrec=QString::fromUtf8(rec);
            //qDebug()<<"第"<<i<<"块"<<"---文件上传="<<mkrec<<endl;
            QJsonParseError jsError;
            QJsonDocument doc(QJsonDocument::fromJson(rec,&jsError));
            if (jsError.error !=QJsonParseError::NoError) {
                netreply->deleteLater();
                break;
            }
            QJsonObject dataMap = doc.object().value("data").toObject();
            QString msgString = dataMap.value("msg").toString();

            if (msgString =="上传成功"){
                m_uploadedBlock = i;
                m_uploadedSize = loadsize;
                QTime endTime = QTime::currentTime();
                int nslapsed = starttime.msecsTo(endTime);
                emit updatepoolloadfileprocess(1,m_strfileuuid, loadsize , m_nfilesize, nslapsed/1000, 0,m_uploadFileInfo);
            } else {
                //qDebug()<<"-----------文件上传接口失败="<<msgString;
                m_bquit = true;
                netreply->deleteLater();
                break;
            }
        } else {
            netreply->deleteLater();
            break;
        }
        delete multiPart;
    }
    if(m_bquit) {
        emit stoploadfileblock(m_strfileuuid, i * fileblocksize);
        return 1;
    } else {
        return 0;
    }
}

int filethread::checksuccessuploadfile()
{
    QVariant cook;
    cook.setValue(disktool->getcookies());

    QString url=NetDiskNetWork_VALUE_mergeEexcerpts;
    QString fileName001 = disktool->URLEncoderWithUrl(m_strfilename);

    QString string000 = QString("md5=%1&fileName=%2&fileSize=%3")
            .arg(m_strfilemd5).arg(fileName001).arg(QString::number(m_nfilesize));
    if(!m_fileToUuid.isEmpty()) {
        string000 = QString("%1&dirUuid=%2").arg(string000).arg(m_fileToUuid);
    }
    if(!m_fileType.isEmpty()) {
        string000 = QString("%1&fileType=%2").arg(string000).arg(m_fileType);
    }

    QByteArray checkparameter=QString("%1").arg(string000).toUtf8();
    //qDebug()<<"-----------合并接口url="<<url<<"----------"<<string000;
    QNetworkRequest netquestfile;
    QSslConfiguration config;
    QSslConfiguration conf = netquestfile.sslConfiguration();
    conf.setPeerVerifyMode(QSslSocket::VerifyNone);
    conf.setProtocol(QSsl::TlsV1SslV3);
    netquestfile.setSslConfiguration(conf);
    netquestfile.setUrl(QUrl(url));
    netquestfile.setRawHeader("Content-Type","application/x-www-form-urlencoded; charset=UTF-8");
    netquestfile.setHeader(QNetworkRequest::CookieHeader,cook);
    QEventLoop waitloop;
    QNetworkReply *reply;
    reply=disktool->getWorkManager()->post(netquestfile,checkparameter);
    connect(reply, &QNetworkReply::finished, &waitloop, &QEventLoop::quit);
    waitloop.exec(QEventLoop::ExcludeUserInputEvents);
    try {
        if (reply->error() == QNetworkReply::NoError) {
            QByteArray rec=reply->readAll();
            QString mkrec=QString::fromUtf8(rec);
            //qDebug()<<"-----------合并接口="<<mkrec;
            QJsonParseError jsError;
            QJsonDocument doc(QJsonDocument::fromJson(rec,&jsError));
            QJsonObject dataMap = doc.object().value("data").toObject();
            QString msgString = dataMap.value("msg").toString();

            QJsonObject fileinfor = dataMap.value("rcFileInfo").toObject();
            QString rfiFileName = fileinfor.value("rfiFileName").toString();
            QString rfiFileType = fileinfor.value("rfiFileType").toString();
            QString fileImageName = disktool->getFileTypeImage(rfiFileType);
            int rfiFileSize = fileinfor.value("rfiFileSize").toInt();
            m_uploadFileInfo.filesize = QString("%1").arg(rfiFileSize);
            m_uploadFileInfo.showSize = disktool->getShowSizeWithFileSize(m_uploadFileInfo.filesize);
            m_uploadFileInfo.filedatetime=fileinfor.value("lastUpdateTime").toString();
            m_uploadFileInfo.fileUuid = fileinfor.value("uuid").toString();
            m_uploadFileInfo.copy = fileinfor.value("copy").toBool();
            //m_uploadFileInfo.radAbsPath = fileinfor.value("radAbsPath").toString();//上传完成后未返回
            //m_uploadFileInfo.radSpecialType = fileinfor.value("radSpecialType").toString().toInt();//上传完成后未返回
            m_uploadFileInfo.isdirectory =0;
            m_uploadFileInfo.filename=rfiFileName+rfiFileType;
            m_uploadFileInfo.name=rfiFileName;
            //m_uploadFileInfo.fileMd5=fileinfor.value("md5").toString();
            m_uploadFileInfo.fileMd5 = m_strfilemd5;
            m_uploadFileInfo.toFileUuid = m_fileToUuid;
            m_uploadFileInfo.fileType = rfiFileType;
            m_uploadFileInfo.filetypeimage =":/image/iconmin/"+fileImageName;
            m_uploadFileInfo.filetypeMaximage =":/image/iconmax/"+fileImageName;
            m_uploadFileInfo.isRecycleFile = 0;

            //qDebug()<<"-----------合并接口="<<msgString;
            if (jsError.error ==QJsonParseError::NoError) {
                m_uploadedSize = 0;
                m_isFinish = 1;
                if (msgString=="合并成功") { 
                    emit updatepoolloadfileprocess(1, m_strfileuuid, m_nfilesize,m_nfilesize, 1,1,m_uploadFileInfo);
                } else {
                    emit updatepoolloadfilestatus(1, m_strfileuuid, 1);
                }
            }
        }
    } catch (QString exception) {
         return 0;
    }
    return 0;
}

void filethread::downloadfile()
{
    getonefileinfo();
    //qDebug()<<"-----------下载文件大小size="<<m_nfilesize<<endl;
    if(m_nfilesize > 0) {
        downloadblock();
    } else {
        m_isFinish = 1;
        //文件已删除
        emit updatepoolloadfilestatus(2, m_strfileuuid, 4);
    }
}

bool filethread::getfilemd5(const QString &strfilepath, QString &strfilemd5)
{
    QFile localFile(strfilepath);
    if (!localFile.open(QFile::ReadOnly))
        return false;
    QCryptographicHash ch(QCryptographicHash::Md5);
    quint64 totalBytes = 0;
    quint64 bytesWritten = 0;
    quint64 bytesToWrite = 0;
    quint64 loadSize = 1024 * 1024 * 50;
    QByteArray buf;
    totalBytes = localFile.size();
    bytesToWrite = totalBytes;
    while (1) {
        if(bytesToWrite > 0) {
            buf = localFile.read(qMin(bytesToWrite, loadSize));
            ch.addData(buf);
            bytesWritten += buf.length();
            bytesToWrite -= buf.length();
            buf.resize(0);
        } else {
            break;
        }
        if(bytesWritten == totalBytes) {
            break;    
        }
    }
    localFile.close();
    strfilemd5 = ch.result().toHex();
    return true;
}

void filethread::getonefileinfo()
{
    QVariant cook;
    cook.setValue(disktool->getcookies());
    QString url=NetDiskNetWork_VALUE_findRcFileInfoByIdList;
    QString string000 = QString("uuid=%1&rfiFileStatus=1").arg(m_strfileuuid);
    QByteArray downloadparameter=QString("%1").arg(string000).toUtf8();
    //qDebug()<<"-----------获取下载文件信息接口url="<<url<<"---------------"<<downloadparameter;

    QNetworkReply *reply;
    QNetworkRequest workquest;
    QSslConfiguration config;
    QSslConfiguration conf = workquest.sslConfiguration();
    conf.setPeerVerifyMode(QSslSocket::VerifyNone);
    conf.setProtocol(QSsl::TlsV1SslV3);
    workquest.setSslConfiguration(conf);
    workquest.setUrl(QUrl(url));
    workquest.setHeader(QNetworkRequest::CookieHeader,cook);
    reply= disktool->getWorkManager()->post(workquest,downloadparameter);
    QEventLoop waitloop;
    connect(reply, &QNetworkReply::finished, &waitloop, &QEventLoop::quit);
    waitloop.exec(QEventLoop::ExcludeUserInputEvents);
    if (reply->error()==QNetworkReply::NoError) {
        QByteArray rec=reply->readAll();
        QString mkrec=QString::fromUtf8(rec);
        //qDebug()<<"-----------获取下载文件信息接口="<<mkrec;
        QJsonParseError jsError;
        QJsonDocument doc(QJsonDocument::fromJson(rec,&jsError));
        if (jsError.error !=QJsonParseError::NoError) {
            return;
        }
        QJsonObject dataMap = doc.object().value("data").toObject();
        int filesize = 0;
        QString rfiFileType = "";
        QString rfiFileName = "";
        QString md5 = "";
        if(dataMap.count()==0) {
            QJsonArray dataArray = doc.object().value("data").toArray();
            if(dataArray.count()>0) {
                QJsonObject map000 = dataArray.at(0).toObject();
                filesize = map000.value("rfiFileSize").toInt();
                md5 = map000.value("md5").toString();
                rfiFileType = map000.value("rfiFileType").toString();
                rfiFileName = map000.value("rfiFileName").toString();
            }
        } else {
            filesize = dataMap.value("rfiFileSize").toInt();
            md5 = dataMap.value("md5").toString();
            rfiFileType = dataMap.value("rfiFileType").toString();
            rfiFileName = dataMap.value("rfiFileName").toString();
        }
        m_downloadFileMD5 = md5;
        m_nfilesize = filesize;
        m_downloadFileType = rfiFileType;
        m_downloadFileName = rfiFileName;
    } else {
        qDebug()<<reply->error()<<endl;
    }
}

void filethread::downloadblock()
{
    createTempFile();//创建下载临时文件夹
    checkSubsectionFileAction();//检查已经下载的文件
    QVariant cook;
    cook.setValue(disktool->getcookies());
    QNetworkRequest netquest000;
    netquest000.setRawHeader("Accept", "application/json");
    netquest000.setRawHeader("Content-Type","application/x-www-form-urlencodedUTF-8");

    QSslConfiguration conf = netquest000.sslConfiguration();
    conf.setPeerVerifyMode(QSslSocket::VerifyNone);
    conf.setProtocol(QSsl::TlsV1SslV3);
    netquest000.setSslConfiguration(conf);

    //m_nfilesize = 0;
    qint64 filesize = m_nfilesize; //文件大小
    qint64 fileReadLocationStart = m_downloadBlockNumber*fileblocksize;
    qint64 fileWriteLocation = fileReadLocationStart;
    qint64 subsectionFileSize = fileblocksize;
    qint64 i = m_downloadBlockNumber;
    qint64 blocks = 1;
    if(fileblocksize > 0) {
        blocks = filesize / fileblocksize;
        if(blocks == 0) {
            blocks = 1;
        } else if (filesize % fileblocksize != 0){
                blocks += 1;
        }
    }
    bool berror = false;
    QTime starttime = QTime::currentTime();
    QString fileName001 = disktool->URLEncoderWithUrl(m_strfilename);
    while ( i < blocks && !m_bquit ) {
        i++;
        if(i == blocks) {
            fileReadLocationStart = fileWriteLocation;
            if(fileblocksize >0) {
                subsectionFileSize = filesize % fileblocksize;
            } else {
                subsectionFileSize = 0;
            }
        } else {
            if( i != 1 && i == (m_ndownloadblock + 1)  ) {
                fileReadLocationStart = i * fileblocksize;
            } else {
                fileReadLocationStart = fileWriteLocation;
            }
        }
        QString currentFilePath = m_tempFilePath+QDir::separator()+downloadTempFileName+QString("%1").arg(i);
        QFile file(QString("%1").arg(currentFilePath));
        file.open(QIODevice::WriteOnly);
        //qDebug()<<"-----文件下载-总共"<<blocks<<"块，第"<<currentFilePath<<"块"<<endl;
        QString string000 = QString("fileUuid=%1&customFileName=%2&offset=%3&length=%4")
                .arg(m_strfileuuid).arg(fileName001).arg(fileReadLocationStart).arg(subsectionFileSize);
        QString url=QString("%1?%2").arg(NetDiskNetWork_VALUE_normalDownload).arg(string000);
        //qDebug()<<"-----文件下载到第"<<i<<"块"<<endl;
        //qDebug()<<"-----文件下载="<<url<<endl;
        netquest000.setUrl(QUrl(url));
        netquest000.setHeader(QNetworkRequest::CookieHeader,cook);
        QNetworkReply *reply;
        QNetworkAccessManager *manager = disktool->getWorkManager();
        reply=manager->get(netquest000);
        QEventLoop waitloop;
        connect(reply, &QNetworkReply::finished, &waitloop, &QEventLoop::quit);
        waitloop.exec(QEventLoop::ExcludeUserInputEvents);
        //qDebug()<<reply->error()<<endl;
        if (reply->error()==QNetworkReply::NoError) {
            QByteArray buffer = reply->readAll();
            file.write(buffer);

            fileWriteLocation = fileReadLocationStart+subsectionFileSize;
            QTime endTime = QTime::currentTime();
            int nslapsed = starttime.msecsTo(endTime);
            emit updatepoolloadfileprocess(2, m_strfileuuid,  fileWriteLocation ,m_nfilesize, nslapsed/1000, 0,m_uploadFileInfo);
        } else {
            m_bquit = true;
            berror = true;
        }
        file.close();
    }
    if(i == blocks) {
        m_isFinish = 1;
        if(berror) {
            emit updatepoolloadfilestatus(2, m_strfileuuid, 2);
        } else {
            if(m_bquit) {
                emit updatepoolloadfilestatus(2, m_strfileuuid, 0);//下载暂停
            } else {
                fileMergeAction();
            }
        }
    } else {
        if(berror){
            m_isFinish = 1;
            emit updatepoolloadfilestatus(2, m_strfileuuid, 2);
        } else {
            emit stoploadfileblock(m_strfileuuid, i);
        }
    }
}

void filethread::fileMergeAction()
{
    QFile downloadfile(m_strfilelocalpath);
    downloadfile.open(QIODevice::ReadWrite);
    int blocks =1;
    if(fileblocksize > 0) {
        blocks = m_nfilesize / fileblocksize;
        if(blocks == 0) {
            blocks = 1;
        } else if (m_nfilesize % fileblocksize != 0) {
            blocks += 1;
        }
    }
    int i = 0;
    //qDebug()<<"-----------下载文件大小size="<<m_nfilesize<<"，共"<<blocks<<"块"<<endl;
    qint64 fileWriteLocationStart = 0;
    while (i < blocks) {
        i++;
        QString currentFilePath = m_tempFilePath+QDir::separator()+downloadTempFileName+QString("%1").arg(i);
        QFile tempfile(currentFilePath);
        tempfile.open(QIODevice::ReadOnly);
        downloadfile.seek(fileWriteLocationStart);
        QByteArray fileData = tempfile.readAll();
        downloadfile.write(fileData);
        //qDebug()<<"---------第"<<i<<"段，本地文件合并写入位置="<<fileWriteLocationStart<<endl;
        if(i == blocks) {
            if(fileblocksize > 0) {
                fileWriteLocationStart = m_nfilesize % fileblocksize;
            } else {
                fileWriteLocationStart = 0;
            }
        } else {
            fileWriteLocationStart = (i) * fileblocksize;
        }
        tempfile.close();
    }
    downloadfile.close();

    deleteDirWithPath(m_tempFilePath);
    //下载完成检验文件MD5
    QString strfilemd5;
    getfilemd5(m_strfilelocalpath, strfilemd5);
    if(m_downloadFileMD5 == strfilemd5) {
        //qDebug()<<"-----下载文件md5校验成功="<<endl;
        emit updatepoolloadfileprocess(2, m_strfileuuid, 0,0, 0,1,m_uploadFileInfo);
    } else {
        //qDebug()<<"-----下载文件md5校验失败="<<endl;
        //qDebug()<<"-----本地文件md5="<<strfilemd5<<endl;
        //qDebug()<<"-----下载文件md5="<<m_downloadFileMD5<<endl;
        emit updatepoolloadfileprocess(2, m_strfileuuid, m_nfilesize,m_nfilesize, 0,0,m_uploadFileInfo);
        emit updatepoolloadfilestatus(2, m_strfileuuid, 1);//校验失败
    }
}

bool filethread::createTempFile()
{
    QString mainPath;
    userDBTool->getTempFilePath(mainPath);//检测临时文件夹是否存在，如果不存在重新创建
    QString uuid = getMD5Encode(m_strfileuuid);
    QString tempfilePath = mainPath+QDir::separator()+uuid;
    QDir homedir;
    if (!homedir.exists(tempfilePath))
        homedir.mkpath(tempfilePath);
    m_tempFilePath = tempfilePath;
    return homedir.mkpath(tempfilePath);
}

void filethread::checkSubsectionFileAction()
{
    m_downloadBlockNumber = 0;
    qint64 fileWriteLocationStart = 0;
    QString tempfilePath = m_tempFilePath;

    QList<int> numberList;
    QDir directory(tempfilePath);
    directory.setFilter(QDir::Files);
    QStringList files=directory.entryList();
    QString tempString = downloadTempFileName;
    for(int x = 0;x < files.count();x++) {
        QString filePath = files[x];
        if(filePath.contains(tempString)) {
            QString numberStr = filePath.replace(tempString,"");
            if(!numberStr.isEmpty()) {
                numberList.append(numberStr.toInt());
            }
        }
    }
    int count = numberList.count();
    int maxTemp = 0;
    if(count >0) {
        maxTemp = numberList.first();
    }
    for(int m = 0;m < count;m++) {
        int number = numberList[m];
        if(maxTemp < number) {
            maxTemp = number;
        }
    }
    //获取已经下载的分段数据中最后一段数据，并比计算这段数据的长度（下载暂停时可能有不完整的数据）
    int blocks = 1;
    if(fileblocksize > 0) {
        blocks = m_nfilesize / fileblocksize;
        if(blocks == 0) {
            blocks = 1;
        } else if (m_nfilesize % fileblocksize != 0) {
            blocks += 1;
        }
    }

    qint64 subsectionFileSize = fileblocksize;
    if(blocks == maxTemp) {
        if(fileblocksize > 0) {
            subsectionFileSize = m_nfilesize % fileblocksize;
        } else {
            subsectionFileSize = 0;
        }
    }
    QString currentFilePath = tempfilePath+QDir::separator()+QString("%1%2").arg(downloadTempFileName).arg(maxTemp);
    //qDebug()<<"----已经下载到="<<currentFilePath;
    QFile tempfile(currentFilePath);
    tempfile.open(QIODevice::ReadOnly);
    if(tempfile.exists()) {
        qint64 tempSize = tempfile.size();
        if(tempSize == 0) {
            if(maxTemp >= 1) {
                fileWriteLocationStart = maxTemp-1;
            } else {
                fileWriteLocationStart = 0;
            }
        } else {
            if(tempSize == subsectionFileSize) {
                fileWriteLocationStart = maxTemp;
            } else {
                fileWriteLocationStart = maxTemp;
            }
        }
    } else {
        fileWriteLocationStart = 0;
    }
    tempfile.close();
    if(fileWriteLocationStart >= 1) {
        m_downloadBlockNumber = fileWriteLocationStart;
    } else {
        m_downloadBlockNumber = 0;
    }
    //qDebug()<<"------------从第"<<fileWriteLocationStart<<"段开始下载"<<endl;
}

