﻿/*
 * 线程 任务 管理类
 */

#include "loadfilethreadpool.h"
#include <QMutexLocker>

loadfilethreadpool::loadfilethreadpool()
{
   m_timer = nullptr;
   maxthreadcount = 2;
}

loadfilethreadpool::~loadfilethreadpool()
{
    if(m_timer)
    {
        delete  m_timer;
        m_timer = nullptr;
    }

    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); ) {
            delete  (*itor);
            *itor = nullptr;
            m_allTask.erase(itor++);
    }
    while(!m_uploadFreeThread.empty()) {
         filethread *pthread = m_uploadFreeThread.front();
         m_uploadFreeThread.pop();
         delete pthread;
    }
    while(!m_downloadFreeThread.empty()) {
         filethread *pthread = m_downloadFreeThread.front();
         m_downloadFreeThread.pop();
         delete pthread;
    }
}

void loadfilethreadpool::createfilethread()
{
    UserDbSession *userdb=UserDbSession::getInstance();
    QString strDownLoadCount;
    userdb->GetSettingsTable(NetDiskSettingsInfomation_KEY_DOWNLOADCOUNT,strDownLoadCount);
    //QString strUpLoadCount;
    //userdb->GetSettingsTable(NetDiskSettingsInfomation_KEY_UPLOADCOUNT,strUpLoadCount);
    maxthreadcount = 3;//默认值
    int nmaxDownLoad = 0;
    if(strDownLoadCount.toInt() > 0)
        nmaxDownLoad = strDownLoadCount.toInt();
    if(nmaxDownLoad != 0)
        maxthreadcount = nmaxDownLoad;
    SetSetingThreadcount(maxthreadcount);

    for (int i = 0; i < maxthreadcount; i++) {
        filethread *uthread = new filethread;
        m_uploadFreeThread.push(uthread);

        filethread *dthread = new filethread;
        m_downloadFreeThread.push(dthread);
    }
    inittime();
    initconnect();
}

void loadfilethreadpool::addtask(loadfiletask *task,int isStart)
{
    // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    QMutexLocker locker(&m_mutex);
    m_allTask.push_back(task);

    if(isStart) {
        qint8 loadType = task->loadtype;
        if(loadType == 5 || loadType == 7) {
            m_uploadWaitTask.push_front(task);
        } else if(loadType == 6 || loadType == 8) {
            m_downloadWaitTask.push_front(task);
        }
    }
}

void loadfilethreadpool::inittime()
{
    if(!m_timer)
        m_timer = new QTimer;
    m_timer->start(1000*2);

    //再加一个 timer 删除task用
    //task 加个标记 close
}

void loadfilethreadpool::initconnect()
{
    connect(m_timer, SIGNAL(timeout()),
            this, SLOT(ontimerout()));
}

bool loadfilethreadpool::isBusy()
{
    bool bIsBusy = false;
     // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end();it++) {
        //loadfiletask *task = it->first;
        filethread *thread = it->second;
        if(!thread->m_bquit) {
            bIsBusy = true;
            break;
        }
    }
    return bIsBusy;
}

void loadfilethreadpool::DelExtraThread()
{
   for (int i = 0; i < (maxthreadcount - m_nsetthreadcount); ) {
       filethread *uthread = m_uploadFreeThread.front();
       m_uploadFreeThread.pop();
       delete uthread;

       filethread *dthread = m_downloadFreeThread.front();
       m_downloadFreeThread.pop();
       delete dthread;
       maxthreadcount --;
   }
}

void loadfilethreadpool::AddExtraThread()
{
    for (int i = 0; i < (m_nsetthreadcount - maxthreadcount); ) {
        filethread *uthread = new filethread;
        m_uploadFreeThread.push(uthread);

        filethread *dthread = new filethread;
        m_downloadFreeThread.push(dthread);

        maxthreadcount ++;
    }
}

void loadfilethreadpool::SetSetingThreadcount(const int &ncount)
{
    m_nsetthreadcount = ncount;
}

int loadfilethreadpool::GetThreadCount()
{
    return  m_nsetthreadcount;
}

void loadfilethreadpool::ontimerout()
{
    if(!m_uploadWaitTask.empty() || !m_downloadWaitTask.empty())
        starttask();
}

void loadfilethreadpool::onfileitemopt(const QString &fileUuid, const int &nType)
{
     // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    if(nType == 5 || nType == 6) {
        pausetask(fileUuid,nType);
    } else if(nType == 1 || nType == 7 || nType == 2 || nType == 8) {
        startpausetask(fileUuid);
    }  else if(nType == -1) {
        canceltask(fileUuid);
    }
}

void loadfilethreadpool::onupdatefileprocess(const int &loadtype, const QString &fileUuid, const qint64 &loadsize, const qint64 &filesize, const int &ntime, const int &type,const Diskfileinformation& fileInfo)
{
    // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
   std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
   for (; it != m_workingTask.end(); it++) {
       loadfiletask *task = it->first;
       //filethread *thread = it->second;
       if(task && task->fileuuid.compare(fileUuid) == 0) {
           task->loadedsize = loadsize;
           emit updatepoolfileprocess( loadtype, fileUuid, loadsize, filesize, ntime, type,fileInfo);
       }
   }
}

void loadfilethreadpool::onthreadfinished()
{
   QMutexLocker locker(&m_mutex);
   std::list<filethread*>::iterator it = m_uploadBusyThread.begin();
   for (; it != m_uploadBusyThread.end(); ) {
       filethread *thread = *it;
       if(thread->m_bquit) {
           thread->m_bquit = true;
           thread->quit();
           thread->wait();
           m_uploadFreeThread.push(*it);
           m_uploadBusyThread.erase(it++);
       } else {
           ++ it;
       }
   }

   std::list<filethread*>::iterator it001 = m_downloadBusyThread.begin();
   for (; it001 != m_downloadBusyThread.end(); ) {
       filethread *thread = *it001;
       if(thread->m_bquit) {
           thread->m_bquit = true;
           thread->quit();
           thread->wait();
           m_downloadFreeThread.push(*it001);
           m_downloadBusyThread.erase(it001++);
       } else {
           ++ it001;
       }
   }
}

void loadfilethreadpool::onupdatepoolloadfilestatus(const int &loadtype, const QString &fileuuid, const int &nstatus)
{
    pausetask(fileuuid,loadtype);
    emit updatepoolfilestatus(loadtype, fileuuid, nstatus);
}

void loadfilethreadpool::onloadfileblock(const QString &fileUuid, const qint64 &block)
{
    foreach(loadfiletask* task, m_allTask) {
        if( task && task->fileuuid.compare(fileUuid)==0 && task->loadedsize < block) {
            task->loadedsize = block;    
        }
    }
}

void loadfilethreadpool::starttask()
{
    // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    QMutexLocker locker(&m_mutex);
    if(m_nsetthreadcount > maxthreadcount)    
        AddExtraThread();
    //上传
    if(!m_uploadWaitTask.empty() && !m_uploadFreeThread.empty()) {
        loadfiletask* task = m_uploadWaitTask.front();
        filethread* thread = m_uploadFreeThread.front();
        if(task && thread) {
            if(task->loadtype == 1 || task->loadtype == 5) {
                 m_uploadWaitTask.pop_front();
            } else if(task->loadtype == 7) {
                connect( thread, SIGNAL(finished()),
                         this, SLOT(onthreadfinished()));
                connect( thread, SIGNAL(updatepoolloadfileprocess(const int&, const QString& , const qint64& ,const qint64&, const int&,const int& ,const Diskfileinformation&)),
                                             this, SLOT(onupdatefileprocess(const int&,const QString& , const qint64& , const qint64& ,const int& ,const int&,const Diskfileinformation&)));
                connect( thread, SIGNAL(updatepoolloadfilestatus(const int &, const QString &, const int &)),
                         this, SLOT(onupdatepoolloadfilestatus(const int &, const QString &, const int &)));
                connect( thread, SIGNAL(stoploadfileblock(const QString& , const qint64& )),
                         this, SLOT(onloadfileblock(const QString&, const qint64&)));
                if(task->loadtype == 7) {
                    task->loadtype = 1;
                }
                thread->addTask(task);
                m_uploadWaitTask.pop_front();
                m_uploadFreeThread.pop();
                m_uploadBusyThread.push_back(thread);
                m_workingTask.insert(std::make_pair(task, thread));
                thread->m_bquit = false;
                thread->start();
            }
        }
    }
    //下载
    if(!m_downloadWaitTask.empty() && !m_downloadFreeThread.empty()) {
        loadfiletask* task = m_downloadWaitTask.front();
        filethread* thread = m_downloadFreeThread.front();
        if(task && thread) {
            if(task->loadtype == 2 || task->loadtype == 6) {
                 m_downloadWaitTask.pop_front();
            } else if(task->loadtype == 8) {
                connect( thread, SIGNAL(finished()),
                         this, SLOT(onthreadfinished()));
                connect( thread, SIGNAL(updatepoolloadfileprocess(const int&, const QString& , const qint64& ,const qint64&, const int&,const int& ,const Diskfileinformation&)),
                                             this, SLOT(onupdatefileprocess(const int&,const QString& , const qint64& , const qint64& ,const int& ,const int&,const Diskfileinformation&)));
                connect( thread, SIGNAL(updatepoolloadfilestatus(const int &, const QString &, const int &)),
                         this, SLOT(onupdatepoolloadfilestatus(const int &, const QString &, const int &)));
                connect( thread, SIGNAL(stoploadfileblock(const QString& , const qint64& )),
                         this, SLOT(onloadfileblock(const QString&, const qint64&)));
                if(task->loadtype == 8) {
                    task->loadtype = 2;
                }
                thread->addTask(task);
                m_downloadWaitTask.pop_front();
                m_downloadFreeThread.pop();
                m_downloadBusyThread.push_back(thread);
                m_workingTask.insert(std::make_pair(task, thread));
                thread->m_bquit = false;
                thread->start();
            }
        }
    }
}

void loadfilethreadpool::pausetask(const QString &strfileuuid,int loadType)
{
    // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    QMutexLocker locker(&m_mutex);

    int type = 0;
    if(loadType == 1 || loadType ==5 || loadType == 7) {
       type = 1;
    }
    if(loadType == 2 || loadType ==6 || loadType == 8) {
        type = 2;
    }
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        loadfiletask *task = it->first;
        filethread *thread = it->second;
        type = thread->m_nType;
        if(task->fileuuid.compare(strfileuuid) == 0) {
            if(task->loadtype == 1 || task->loadtype == 7 )
                task->loadtype = 5;
            else if(task->loadtype == 2 || task->loadtype == 8)
                task->loadtype = 6;
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_workingTask.erase(it++);
        } else {
            it++;
        }
    }
    if(type == 1) {
        std::deque<loadfiletask*>::iterator upitor = m_uploadWaitTask.begin();
        for (; upitor != m_uploadWaitTask.end(); ) {
            if((*upitor)->fileuuid.compare(strfileuuid) == 0) {
                m_uploadWaitTask.erase(upitor++);
            } else {
                ++ upitor;
            }
        }
        std::list<filethread*>::iterator upbit = m_uploadBusyThread.begin();
        for ( ; upbit != m_uploadBusyThread.end(); ) {
            filethread *thread = *upbit;
            if(thread->m_strfileuuid.compare(strfileuuid) == 0) {
                thread->m_bquit = true;
                thread->quit();
                thread->wait();
                m_uploadFreeThread.push(thread);
                m_uploadBusyThread.erase(upbit++);
            } else {
                upbit++;
            }
        }
    } else if(type == 2) {
        std::deque<loadfiletask*>::iterator downitor = m_downloadWaitTask.begin();
        for (; downitor != m_downloadWaitTask.end(); ) {
            if((*downitor)->fileuuid.compare(strfileuuid) == 0) {
                m_downloadWaitTask.erase(downitor++);
            } else {
                ++ downitor;
            }
        }
        std::list<filethread*>::iterator downbit = m_downloadBusyThread.begin();
        for ( ; downbit != m_downloadBusyThread.end(); ) {
            filethread *thread = *downbit;
            if(thread->m_strfileuuid.compare(strfileuuid) == 0) {
                thread->m_bquit = true;
                thread->quit();
                thread->wait();
                m_downloadFreeThread.push(thread);
                m_downloadBusyThread.erase(downbit++);
            } else {
                downbit++;
            }
        }
    }
}

void loadfilethreadpool::canceltask(const QString &strfileuuid)
{
    QMutexLocker locker(&m_mutex);
    int type = 0;
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        filethread *thread = it->second;
        type = thread->m_nType;
        if(it->first->fileuuid.compare(strfileuuid) == 0) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_workingTask.erase(it++);
        } else {
            it++;
        }
    }
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); ) {
        if((*itor)->fileuuid.compare(strfileuuid) == 0) {
            emit delonepooltask((*itor)->loadtype);
            delete  (*itor);
            *itor = nullptr;
            m_allTask.erase(itor++);
        } else {
            ++ itor;
        }
    }
    if(type == 1) {
        std::deque<loadfiletask*>::iterator upitor = m_uploadWaitTask.begin();
        for (; upitor != m_uploadWaitTask.end(); ) {
            if((*upitor)->fileuuid.compare(strfileuuid) == 0) {
                m_uploadWaitTask.erase(upitor++);
            } else {
                ++ upitor;
            }
        }
    } else if(type == 2) {
        std::deque<loadfiletask*>::iterator downitor = m_downloadWaitTask.begin();
        for (; downitor != m_downloadWaitTask.end(); ) {
            if((*downitor)->fileuuid.compare(strfileuuid) == 0) {
                m_downloadWaitTask.erase(downitor++);
            } else {
                ++ downitor;
            }
        }
    }
    if(m_nsetthreadcount < maxthreadcount) {
        DelExtraThread();
    }
}

void loadfilethreadpool::startpausetask(const QString &strfileuuid)
{
    // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    QMutexLocker locker(&m_mutex);
    foreach(loadfiletask *task, m_allTask) {
        if(task->fileuuid.compare(strfileuuid) == 0) {
            if(task->loadtype == 5) {
                task->loadtype = 7;
                m_uploadWaitTask.push_front(task);
            } else if(task->loadtype == 6) {
                task->loadtype = 8;
                m_downloadWaitTask.push_front(task);
            }
        }
    }
}

void loadfilethreadpool::canceltask(const int &type)
{
    if(type == file_uploading) {
        canceluploadtask();
    } else if(type == file_downloading) {
        canceldownloadtask();
    } else {
        cancelalltask();
    }
}

void loadfilethreadpool::closepool()
{
    if (m_timer->isActive() )    
        m_timer->stop();

    clearwaittask(m_uploadWaitTask);
    clearwaittask(m_downloadWaitTask);
    cancelalltask();
    pauseallbusythread();
}

void loadfilethreadpool::startalluploadtask()
{
    // 1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载，9秒传
    //alltask 中暂停的任务 改成 等待状态
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        loadfiletask *task = *itor;
        if(task->loadtype == 5) {
            task->loadtype = 7;
            m_uploadWaitTask.push_front(task);
            emit startLoadTask(5);
        }
    }
}

void loadfilethreadpool::cancelalluploadtask()
{
    canceltask(file_uploading);
}

void loadfilethreadpool::startalldownloadtask()
{
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        loadfiletask *task = *itor;
        if(task->loadtype == 6) {
            task->loadtype = 8;
            m_downloadWaitTask.push_front(task);
            emit startLoadTask(6);
        }
    }  
}

void loadfilethreadpool::cancelalldownloadtask()
{
    canceltask(file_downloading);
}

void loadfilethreadpool::canceluploadtask()
{
    //1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载
    QMutexLocker locker(&m_mutex);
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        loadfiletask *task = (*it).first;
        filethread *thread = it->second;
        if(task->loadtype == 1 || task->loadtype == 5 || task->loadtype == 7) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_workingTask.erase(it++);
        } else {
            ++ it;
        }
    }
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        loadfiletask *task = *itor;
        if(task->loadtype == 1 || task->loadtype == 7) {
            task->loadtype = 5;
        }
    }
    clearwaittask(m_uploadWaitTask);
}

void loadfilethreadpool::canceldownloadtask()
{
    //1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载
    QMutexLocker locker(&m_mutex);
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        loadfiletask *task = (*it).first;
        filethread *thread = it->second;
        if(task->loadtype == 2 || task->loadtype == 6 || task->loadtype == 8) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_workingTask.erase(it++);
        } else {
            ++ it;
        }
    }
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        loadfiletask *task = *itor;
        if(task->loadtype == 2 || task->loadtype == 8) {
            (*itor)->loadtype = 6;    
        }
    }
    clearwaittask(m_downloadWaitTask);
}

void loadfilethreadpool::cancelalltask()
{
    QMutexLocker locker(&m_mutex);
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        filethread *thread = it->second;
        thread->m_bquit = true;
        thread->quit();
        thread->wait();
        m_workingTask.erase(it++);
    }
   //1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        loadfiletask *task =*itor;
        if(task->loadtype == 1 || task->loadtype == 7)
            task->loadtype = 5;
        else if((*itor)->loadtype == 2 || task->loadtype == 8)
            task->loadtype = 6;
    }
}

void loadfilethreadpool::pauseAllThread()
{
    pauseallbusythread();
    cancelalltask();
    clearwaittask(m_uploadWaitTask);
    clearwaittask(m_downloadWaitTask);
}

void loadfilethreadpool::pauseAllUpThread()
{
    pauseAllUpBusyThread();
    closeAllUpTask();
    clearwaittask(m_uploadWaitTask);
}

void loadfilethreadpool::pauseAllDownThread()
{
    pauseAllDownBusyThread();
    closeAllDownTask();
    clearwaittask(m_downloadWaitTask);
}

void loadfilethreadpool::clearwaittask(std::deque<loadfiletask*>& queue) {
    queue.clear();

}

void loadfilethreadpool::pauseallbusythread()
{
    QMutexLocker locker(&m_mutex);
    std::list<filethread*>::iterator upIt = m_uploadBusyThread.begin();
    for ( ; upIt != m_uploadBusyThread.end(); ) {
        filethread *thread = *upIt;
        thread->m_bquit = true;
        thread->quit();
        thread->wait();
        m_uploadFreeThread.push(thread);
        m_uploadBusyThread.erase(upIt++);
    }
    std::list<filethread*>::iterator downIt = m_downloadBusyThread.begin();
    for ( ; downIt != m_downloadBusyThread.end(); ) {
        filethread *thread = *downIt;
        thread->m_bquit = true;
        thread->quit();
        thread->wait();
        m_downloadFreeThread.push(thread);
        m_downloadBusyThread.erase(downIt++);
    }
}

void loadfilethreadpool::closeAllUpTask()
{
    QMutexLocker locker(&m_mutex);
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        loadfiletask *task = it->first;
        filethread *thread = it->second;
        if(task->loadtype == 1 || task->loadtype == 7) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_workingTask.erase(it++);
        } else {
            it++;
        }
    }
   //1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        loadfiletask *task = (*itor);
        if(task->loadtype == 1 || task->loadtype == 7)
            task->loadtype = 5;
    }
}

void loadfilethreadpool::pauseAllUpBusyThread()
{
    QMutexLocker locker(&m_mutex);
    std::list<filethread*>::iterator it = m_uploadBusyThread.begin();
    for ( ; it != m_uploadBusyThread.end(); ) {
        filethread *thread = *it;
        if(thread->m_nType == 1 || thread->m_nType == 7) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_uploadFreeThread.push(thread);
            m_uploadBusyThread.erase(it++);
        } else {
            it++;
        }
    }
}

void loadfilethreadpool::closeAllDownTask()
{
    QMutexLocker locker(&m_mutex);
    std::map<loadfiletask*, filethread*>::iterator it = m_workingTask.begin();
    for ( ; it != m_workingTask.end(); ) {
        loadfiletask *task = it->first;
        filethread *thread = it->second;
        if(task->loadtype == 2 || task->loadtype == 8) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_workingTask.erase(it++);
        } else {
            it++;
        }
    }
   //1 正在上传 2 正在下载 3 上传已完成 4 下载已完成 5 暂停 上传 6 暂停 下载 7 等待上传 8 等待下载
    std::list<loadfiletask*>::iterator itor = m_allTask.begin();
    for (; itor != m_allTask.end(); itor++) {
        if((*itor)->loadtype == 2 || (*itor)->loadtype == 8)
            (*itor)->loadtype = 6;
    }
}

void loadfilethreadpool::pauseAllDownBusyThread()
{   
    QMutexLocker locker(&m_mutex);
    std::list<filethread*>::iterator it = m_downloadBusyThread.begin();
    for ( ; it != m_downloadBusyThread.end(); ) {
        filethread *thread = *it;
        if(thread->m_nType == 2 || thread->m_nType == 8) {
            thread->m_bquit = true;
            thread->quit();
            thread->wait();
            m_downloadFreeThread.push(thread);
            m_downloadBusyThread.erase(it++);
        } else {
            it++;
        }
    }
}
