﻿#ifndef LOADFILETHREADPOOL_H
#define LOADFILETHREADPOOL_H
#include <QObject>
#include "filethread.h"
#include <queue>
#include <list>
#include <map>
#include <QTimer>
#include <QMutex>
#include <QDebug>
#include "UserInformation.h"
#include "userdbsession.h"

class loadfilethreadpool : public QObject
{
    Q_OBJECT
public:
    loadfilethreadpool();
    ~loadfilethreadpool();

    int maxthreadcount;

    void    createfilethread();
    //isStart任务是否自动开启，0是不启动，1是开启；加载本地存储的任务时isStart=0
    void    addtask(loadfiletask* task,int isStart);
    void    inittime();
    void    initconnect();
    bool    isBusy();
    void    DelExtraThread();
    void    AddExtraThread();
    void    SetSetingThreadcount(const int &ncount);
    int     GetThreadCount();

    void    canceltask(const int &type);
    void    closepool();
    void    startalluploadtask();
    void    cancelalluploadtask();
    void    startalldownloadtask();
    void    cancelalldownloadtask();

    void    pauseAllThread();

    void    pauseAllUpThread();
    void    pauseAllDownThread();

    void    startpausetask(const QString& strfileuuid);
    void    canceltask(const QString& strfileuuid);//删除某个任务

private:
    int     m_nsetthreadcount;

    std::queue<filethread*>              m_uploadFreeThread;
    std::list<filethread*>               m_uploadBusyThread;
    std::deque<loadfiletask*>            m_uploadWaitTask;

    std::queue<filethread*>              m_downloadFreeThread;
    std::list<filethread*>               m_downloadBusyThread;
    std::deque<loadfiletask*>            m_downloadWaitTask;

    std::list<loadfiletask*>             m_allTask;
    std::map<loadfiletask*, filethread*> m_workingTask;
    QMutex                               m_mutex;
    QTimer                              *m_timer;

    void    starttask();
    void    pausetask(const QString& strfileuuid,int loadType);

    void    canceluploadtask();
    void    canceldownloadtask();

    void    clearwaittask(std::deque<loadfiletask*>& queue);

    void    cancelalltask();
    void    pauseallbusythread();

    void    closeAllUpTask();
    void    pauseAllUpBusyThread();

    void    closeAllDownTask();
    void    pauseAllDownBusyThread();
private slots:
    void    ontimerout();

signals:
     void   updatewaitforfreethread();
     void   updatepoolfileprocess( const int &loadtype, const QString &fileUuid,
                                  const qint64 &loadedsize,const qint64 &filesize,
                                  const int &ntime ,const int& type,const Diskfileinformation& fileInfo);
     void   updatepoolfilestatus(const int &loadtype, const QString &fileuuid, const int &nstatus);
     void   delonepooltask(const int &loadtype);

     void   startLoadTask(const int &loadtype);

public slots:
    void    onthreadfinished();
    void    onfileitemopt(const QString &fileUuid, const int &nType);//暂停某个任务
    void    onupdatefileprocess(const int& loadtype, const QString &fileUuid, const qint64 & loadsize,
                                const qint64 &filesize,const int& ntime,
                                const int& type,const Diskfileinformation& fileInfo);
    void    onupdatepoolloadfilestatus(const int &loadtype, const QString &fileuuid, const int &nstatus);
    void    onloadfileblock( const QString &fileUuid, const qint64& block);


private:

};

#endif // LOADFILETHREADPOOL_H
